using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MotionTracking;
using System.Threading;
using GestionBluetooth;
using WiimoteLib;

namespace WindowsApplication3
{
    public partial class Form1 : Form
    {
        #region Variables declaration
        private Camera fingersTracking;
        private WiiMoteConnection wiiMoteConnection;
        private Wiimote[] wiimote;
        private HeadTracking headTracking;
        private PointF[][] previousFramesPoint;
        private double previousDis = 0;
        #endregion

        #region Delegates and events declaration

        /// <summary>
        /// Delegate used to call updateTextBox.
        /// </summary>
        /// <param name="text"></param>
        private delegate void updateTextBoxDelegate(string controlToUpdate, string text);



        #endregion

        #region Methods declaration

        /// <summary>
        /// Form constructor
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //Creation of an HeadTracking object to process the data retrieved from the WiiMote
            headTracking = new HeadTracking(Math.PI * 44 / (double)180, ResolutionWidth._1024, ResolutionHeight._768, new Point3DF(0, 0, 0), new Point3DF((float)0.8, 0, 0));
            previousFramesPoint = new PointF[2][];

            for (int i = 0; i < 2; i++)
            {
                previousFramesPoint[i] = new PointF[6];
                for (int j = 0; j < 6; j++)
                {
                    previousFramesPoint[i][j] = new PointF(0, 0);
                }
            }
        }

        /// <summary>
        /// Connect the available WiiMotes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonConnectWiiMote_Click(object sender, EventArgs e)
        {
            //WiiMote Connection
            wiiMoteConnection = new WiiMoteConnection();
            wiiMoteConnection.WiiMoteConnectionChanged += new WiiMoteConnection.WiiMoteConnectionChangedHandler(OnWiiMoteConnectionUpdated);

            Thread wiiMoteConnectionThread = new Thread(new ThreadStart(wiiMoteConnection.Connect));
            wiiMoteConnectionThread.Start();
        }

        /// <summary>
        /// Updates the defined textBox text property.
        /// </summary>
        /// <param name="text"></param>
        private void updateTextBox(string controlToUpdate, string text)
        {
            this.Controls.Find(controlToUpdate, false)[0].Text = text;
        }

        /// <summary>
        /// Processes any change of the fingers position 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnFingersPositionUpdated(object sender, OnCameraFingerChangedEventArgs e)
        {

            //     pictureBox1.Image = e.Image;
            System.Drawing.Point pointA;
            System.Drawing.Point pointB;

            if (e.DiodesPositionList.Count > 0)
            {
                pointA = e.DiodesPositionList[0];
                if (textBoxFingerTracking1.InvokeRequired)
                {

                    textBoxFingerTracking1.BeginInvoke(new updateTextBoxDelegate(updateTextBox), new object[] { "textBoxFingerTracking1", " x : " + pointA.X + " y : " + pointA.Y });
                }
                else
                {
                    textBoxFingerTracking1.Text = " x : " + pointA.X + " y : " + pointA.Y;
                }
                if (e.DiodesPositionList.Count > 1)
                {
                    pointB = e.DiodesPositionList[1];
                    if (textBoxFingerTracking2.InvokeRequired)
                    {
                        textBoxFingerTracking2.BeginInvoke(new updateTextBoxDelegate(updateTextBox), new object[] { "textBoxFingerTracking2", " x : " + pointB.X + " y : " + pointB.Y });
                    }
                    else
                    {
                        textBoxFingerTracking2.Text = " x : " + pointB.X + " y : " + pointB.Y;
                    }
                }
            }
        }

        /// <summary>
        /// Updates the defined textBox text property.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnFramesPositionUpdated(object sender, OnCameraFramesChangedEventArgs e)
        {
            System.Drawing.PointF pointA;
            System.Drawing.PointF pointB;

            if (e.FramesDiodesPositionList.Count == 2)
            {
                pointA = e.FramesDiodesPositionList[0];
                pointB = e.FramesDiodesPositionList[1];

                for (int i = 4; i >= 0; i--)
                {
                    previousFramesPoint[0][i + 1] = previousFramesPoint[0][i];
                }
                previousFramesPoint[0][0] = pointA;

                for (int i = 4; i >= 0; i--)
                {
                    previousFramesPoint[1][i + 1] = previousFramesPoint[1][i];
                }
                previousFramesPoint[1][0] = pointB;

                //pointA.X = 0;
                //pointB.X =0;

                //for (int i = 0; i < 6; i++)
                //{
                //    pointA.X += previousFramesPoint[0][i].X;
                //    pointB.X += previousFramesPoint[1][i].X;
                //}
                //pointA.X /= 6;
                //pointB.X /= 6;
                double distance = headTracking.ProcessDisToWiiMotes(pointA, pointB);
                if (Math.Abs((distance - previousDis) / previousDis) < 0.03) distance = previousDis;
                else previousDis = distance;

                //  if (headTracking != null) Console.Out.WriteLine("distance : " + distance + " dif : " + ((int)pointA.X - pointB.X));

                if (textBoxFingerTracking1.InvokeRequired)
                {
                    textBoxFingerTracking1.BeginInvoke(new updateTextBoxDelegate(updateTextBox), new object[] { "textBoxFramesTracking1", " x : " + pointA.X + " y : " + pointA.Y });
                    textBoxFingerTracking1.BeginInvoke(new updateTextBoxDelegate(updateTextBox), new object[] { "textBoxFramesTracking2", " x : " + pointB.X + " y : " + pointB.Y });
                    textBoxFingerTracking1.BeginInvoke(new updateTextBoxDelegate(updateTextBox), new object[] { "textBoxFramesTrackingDis", distance.ToString() });
                }
                else
                {
                    textBoxFramesTracking1.Text = " x : " + pointA.X + " y : " + pointA.Y;
                    textBoxFramesTracking2.Text = " x : " + pointB.X + " y : " + pointB.Y;
                }
            }
        }

        /// <summary>
        /// Processes any modification of the WiiMotes connections status
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnWiiMoteConnectionUpdated(object sender, OnWiiMoteConnectionChangedEventArgs e)
        {
        }

        /// <summary>
        /// Disconnects the infrared cameras and the WiiMotes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            //    fingersTracking.Dispose();
            if (fingersTracking != null) fingersTracking.Dispose();
            ////Disconnect the camera
            //if (fingersTracking != null)
            //{
            //    fingersTracking.CameraFingerChanged -= new Camera.CameraFingerChangedHandler(OnFingersPositionUpdated);
            //    fingersTracking.Disconnect();
            //}

            //Disconnect the WiiMotes
            if (wiiMoteConnection != null)
            {
                if (wiimote != null)
                {
                    for (int i = 0; i < wiimote.Length; i++)
                    {
                        wiimote[i].Disconnect();
                        wiimote[i].WiimoteChanged -= new EventHandler<WiimoteChangedEventArgs>(OnWiiMoteChanged);
                    }
                }

                wiiMoteConnection.WiiMoteConnectionChanged -= new WiiMoteConnection.WiiMoteConnectionChangedHandler(OnWiiMoteConnectionUpdated);
                wiiMoteConnection.Disconnect();
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            //Retrieve the number of Wiimotes connected to the computer
            WiimoteCollection wiiMoteCollection = new WiimoteCollection();
            try
            {
                wiiMoteCollection.FindAllWiimotes();
                labelNbWiiMote.Text = wiiMoteCollection.Count.ToString();
            }
            catch (Exception wiiMoteDetectionException)
            {
                MessageBox.Show("No WiiMotes have been found");
            }


            //If two wiimotes at least have been found, add them to a wiimotes array
            if (wiiMoteCollection.Count > 0)
            {
                wiimote = new Wiimote[1];
                for (int i = 0; i < 1; i++)
                {
                    wiimote[i] = wiiMoteCollection[i];
                    wiimote[i].Connect();
                    wiimote[i].SetReportType(InputReport.IRAccel, true);
                    wiimote[i].SetLEDs(false, false, false, false);
                    wiimote[i].WiimoteChanged += new EventHandler<WiimoteChangedEventArgs>(OnWiiMoteChanged);
                }
            }
            //Creation of an HeadTracking object to process the data retrieved from the WiiMote
            headTracking = new HeadTracking(Math.PI * 44 / (double)180, ResolutionWidth._1024, ResolutionHeight._768, new Point3DF(0, 0, 0), new Point3DF((float)0.8, 0, 0));

        }

        private void button2_Click(object sender, EventArgs e)
        {
            //Camera initialization
            //Do not forget to disable the automatic mode of the exposure option of the webcam with a tool like VidCap (Vidcap32.exe)
            fingersTracking = new Camera(ResolutionWidth._640, ResolutionHeight._480, ResolutionWidth._1024, ResolutionHeight._768);
            fingersTracking.Connect();
            //  fingersTracking.InverseYAxisState = true;
            fingersTracking.CameraFingerChanged += new Camera.CameraFingerChangedHandler(OnFingersPositionUpdated);
            fingersTracking.CameraFingerLost += new Camera.CameraFingerLostHandler(OnFingersDiodeLost);
            fingersTracking.CameraFramesChanged += new Camera.CameraFramesChangedHandler(OnFramesPositionUpdated);
            fingersTracking.HeadTrackingState = true;
            fingersTracking.TrackInfraredDiodes();

        }

        private void OnFingersDiodeLost(object sender, OnCameraFingerLostEventArgs e)
        {
            Console.Out.WriteLine("lost" + e.DiodeLostId);
        }

        /// <summary>
        /// Handles the position changed events detected by the two wiimotes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnWiiMoteChanged(object sender, WiimoteChangedEventArgs e)
        {
            System.Drawing.Point[][] wiiMoteLedsFound = new System.Drawing.Point[2][];
            wiiMoteLedsFound[0] = new System.Drawing.Point[4];
            wiiMoteLedsFound[1] = new System.Drawing.Point[4];

            //Retrieve the coordinates of the leds found by the wiimote1
            IRSensor[] irSensors = wiimote[0].WiimoteState.IRState.IRSensors;
            for (int i = 0; i < 4; i++)
            {
                if (irSensors[i].Found == true) wiiMoteLedsFound[0][i] = new System.Drawing.Point(irSensors[i].RawPosition.X, irSensors[i].RawPosition.Y);
                else wiiMoteLedsFound[0][i] = new System.Drawing.Point(-1, -1);
            }


            //Remove the leds position which do not indicate the coordinates of the frame
            if (headTracking != null)
            {
                Point[][] framesLedPosition = headTracking.GetFramesLeds(wiiMoteLedsFound[0]);

                if (framesLedPosition.GetLength(0) > 0 && framesLedPosition[0].GetLength(0) == 2)
                {
                    if (textBoxWiiMote1D1.InvokeRequired)
                    {
                        //     Console.Out.WriteLine("distance" +       headTracking.ProcessDisToWiiMotes(framesLedPosition[0][0], framesLedPosition[0][1]));

                        textBoxWiiMote1D1.BeginInvoke(new updateTextBoxDelegate(updateTextBox), new Object[] { "textBoxWiiMote1D1", " x : " + framesLedPosition[0][0].X + " y : " + framesLedPosition[0][0].Y });
                        textBoxWiiMote1D1.BeginInvoke(new updateTextBoxDelegate(updateTextBox), new Object[] { "textBoxWiiMote1D2", " x : " + framesLedPosition[0][1].X + " y : " + framesLedPosition[0][1].Y });
                        textBoxWiiMote1D1.BeginInvoke(new updateTextBoxDelegate(updateTextBox), new Object[] { "textBoxWiiMote1Dif", " Dif " + ((int)framesLedPosition[0][0].X - framesLedPosition[0][1].X) });
                    }
                    else
                    {
                        textBoxWiiMote1D1.Text = " x : " + framesLedPosition[0][0].X + " y : " + framesLedPosition[0][0].Y;
                        textBoxWiiMote1D2.Text = " x : " + framesLedPosition[0][1].X + " y : " + framesLedPosition[0][1].Y;
                    }
                }

                Console.Out.WriteLine(headTracking.Get3DCoordArray(wiiMoteLedsFound[0], wiiMoteLedsFound[1])[3].Z);
            }
        }

        private void Form1_Resize(object sender, EventArgs e)
        {

        }

        #endregion
    }
}