using System;
using System.Collections.Generic;
using System.Text;
using GestionBluetooth;
using System.Windows.Forms;

namespace WindowsApplication3
{
    /// <summary>
    /// Manages the WiiMotes connections
    /// </summary>
    class WiiMoteConnection
    {
        
        #region Delegates & events declaration

        /// <summary>
        /// WiiMoteConnectionChanged event handler
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public delegate void WiiMoteConnectionChangedHandler(object sender, OnWiiMoteConnectionChangedEventArgs e);

        /// <summary>
        /// WiiMoteConnectionChanged event 
        /// </summary>
        public event WiiMoteConnectionChangedHandler WiiMoteConnectionChanged;

        #endregion

        #region Methods declaration

        /// <summary>
        /// Launches the WiiMotes discovery and connect to the found WiiMotes
        /// </summary>
        public void Connect()
        {
            DialogResult result = MessageBox.Show("Press simultaneously on the \"1\" and \"2\" buttons of the wiimotes", "WiiMotes connection", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);

            //Get a quick list of the bluetooth devices connected to the computer
            BluetoothPeriph wiiMoteDevices = new BluetoothPeriph(new IntPtr(), 1);
            int nbWiiMoteConnected = 0;
            for (int i = 0; i < wiiMoteDevices.ListePeriph.Count; i++)
            {
                if (wiiMoteDevices.ListePeriph[i].szName.Equals("Nintendo RVL-CNT-01"))
                {
                    //In case a wiimote is remembered but not connected : it is easier to remove the device and to connect to it again
                    if (wiiMoteDevices.ListePeriph[i].fRemembered == true && wiiMoteDevices.ListePeriph[i].fConnected == false)
                    {
                        try
                        {
                            wiiMoteDevices.DeconnecterPeripherique(new IntPtr(), wiiMoteDevices.ListePeriph[i]);
                        }
                        catch
                        {
                            Console.Out.WriteLine("Error during the WiiMote disconnection");
                        }
                    }
                    //Count the WiiMotes already connected to the computer
                    else if (wiiMoteDevices.ListePeriph[i].fConnected == true) nbWiiMoteConnected++;
                }
            }

            //WiiMote connection
            int cptConnectionTrials = 0;

                      while (result == DialogResult.OK && nbWiiMoteConnected < 2 && cptConnectionTrials < 2)
            {
                nbWiiMoteConnected = 0;
                wiiMoteDevices = new BluetoothPeriph(new IntPtr(), 7);
                //In case a wiimote is remembered but not connected : it is easier to remove the device and to connect to it again
                for (int i = 0; i < wiiMoteDevices.ListePeriph.Count; i++)
                {
                    if (wiiMoteDevices.ListePeriph[i].szName.Equals("Nintendo RVL-CNT-01"))
                    {

                        if (wiiMoteDevices.ListePeriph[i].fRemembered == false && wiiMoteDevices.ListePeriph[i].fConnected == false)
                        {
                            try
                            {
                                wiiMoteDevices.ConnecterPeripherique(new IntPtr(), wiiMoteDevices.ListePeriph[i], new Guid("00001124-0000-1000-8000-00805F9B34FB"));
                            }
                            catch
                            {
                                Console.Out.WriteLine("Error during the WiiMote connection");
                            }
                        }
                        if (wiiMoteDevices.ListePeriph[i].fConnected == true) nbWiiMoteConnected++;
                    }
                }
                cptConnectionTrials++;

                if (cptConnectionTrials > 1 && nbWiiMoteConnected < 2)
                    result = MessageBox.Show("Press simultaneously on the \"1\" and \"2\" buttons of the wiimotes", "WiiMotes connection", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);
            }
        }

        /// <summary>
        /// Launches the WiiMotes discovery and disconnect the found WiiMotes
        /// </summary>
        public void Disconnect()
        {
            //Gets a quick list of the bluetooth devices connected to the computer
            BluetoothPeriph wiiMoteDevices = new BluetoothPeriph(new IntPtr(), 1);
             int i = 0;
            while (wiiMoteDevices.ListePeriph.Count > 0 &&  i< wiiMoteDevices.ListePeriph.Count)
            {
                if (wiiMoteDevices.ListePeriph[i].szName.Equals("Nintendo RVL-CNT-01"))
                {
                    //In case a wiimote is remembered but not connected : it is easier to remove the device and to connect to it again
                    if (wiiMoteDevices.ListePeriph[i].fRemembered == true || wiiMoteDevices.ListePeriph[i].fConnected == true)
                    {
                        try
                        {
                            wiiMoteDevices.DeconnecterPeripherique(new IntPtr(), wiiMoteDevices.ListePeriph[i]);
                        }
                        catch
                        {
                            Console.Out.WriteLine("Error during the WiiMote disconnection");
                        }
                    }
                    else i++;
                }
            }
        }

        /// <summary>
        /// Raises a WiiMoteConnectionChanged event
        /// </summary>
        /// <param name="e"></param>
        public void RaiseWiiMoteConnectionChangedEvent(OnWiiMoteConnectionChangedEventArgs e)
        {
            if (WiiMoteConnectionChanged != null)
            {
                lock (WiiMoteConnectionChanged)
                {
                    WiiMoteConnectionChanged(this, e);
                }
            }
        }

        #endregion

    }

    /// <summary>
    /// WiiMote connection state changed event class
    /// </summary>
    class OnWiiMoteConnectionChangedEventArgs : EventArgs
    {
        #region Variables declaration

        int nbWiiMoteConnected;

        #endregion

        #region Methods declaration
       
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="nbWiiMoteConnectedTemp"></param>
        public OnWiiMoteConnectionChangedEventArgs(int nbWiiMoteConnectedTemp)
        {
            nbWiiMoteConnected = nbWiiMoteConnectedTemp;
        }
       
#endregion

        #region Properties declaration

        /// <summary>
        /// Gets the number of WiiMote connected to the computer
        /// </summary>
        public int NbWiiMoteConnected
        {
            get { return nbWiiMoteConnected; }
        }

        #endregion


    }
}
