using System;
using System.Collections.Generic;
using System.Text;
using Emgu.CV;
using Emgu.CV.Structure;
using System.Drawing;

namespace MotionTracking
{
    /// <summary>
    /// Diode found event class.
    /// </summary>
    public class OnCameraFingerFoundEventArgs : EventArgs
    {
        #region Variables declaration

        private byte diodeFoundId;
        private Point diodeFoundPosition;

        #endregion

        #region Methods declaration

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="diodeFoundIdTemp"></param>
        /// <param name="diodeFoundPositionTemp"></param>
        public OnCameraFingerFoundEventArgs(byte diodeFoundIdTemp, Point diodeFoundPositionTemp)
        {
            diodeFoundId = diodeFoundIdTemp;
            diodeFoundPosition = diodeFoundPositionTemp;
        }
        #endregion

        #region Properties declaration

        /// <summary>
        /// Get the id of the found diode.
        /// </summary>
        public byte DiodeFoundId
        {
            get { return diodeFoundId; ;}
        }

        /// <summary>
        /// Get the position of the found diode.
        /// </summary>
        public Point DiodeFoundPosition
        {
            get { return diodeFoundPosition; }
        }
        #endregion
    }

    /// <summary>
    /// Diodes position changed event class.
    /// </summary>
    public class OnCameraFingerChangedEventArgs : EventArgs
    {
        #region Variables declaration
        private List<Point> diodesPositionList;
        #endregion

        #region Methods declaration
        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="diodesPositionListTemp"></param>
        public OnCameraFingerChangedEventArgs( List<Point> diodesPositionListTemp)
        {
            diodesPositionList = diodesPositionListTemp;
             }
        #endregion

        #region Properties declaration

        /// <summary>
        /// Get the list of the positions of the fingers diodes tracked.
        /// </summary>
        public List<Point> DiodesPositionList
        {
            get { return diodesPositionList; }
        }

               #endregion

    }

    /// <summary>
    /// Diode lost event class.
    /// </summary>
    public class OnCameraFingerLostEventArgs : EventArgs
    {
        #region Variables declaration

        private byte diodeLostId;
        private Point diodeLastPosition;

        #endregion

        #region Methods declaration

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="diodeLostIdTemp"></param>
        /// <param name="diodeLastPositionTemp"></param>
        public OnCameraFingerLostEventArgs(byte diodeLostIdTemp, Point diodeLastPositionTemp)
        {
            diodeLostId = diodeLostIdTemp;
            diodeLastPosition = diodeLastPositionTemp;
        }
        #endregion

        #region Properties declaration

        /// <summary>
        /// Get the id of the lost diode.
        /// </summary>
        public byte DiodeLostId
        {
            get { return diodeLostId; }
        }

        /// <summary>
        /// Get the last position detected before loosing the diode.
        /// </summary>
        public Point DiodeLastPosition
        {
            get { return diodeLastPosition; }
        }

        #endregion
    }

    /// <summary>
    /// Diodes position changed event class.
    /// </summary>
    /// <remarks>ONLY USED FOR DEBUGGING (create a bitmap for each event)</remarks>
        public class OnCameraFingerChangedDebugEventArgs : EventArgs
    {
        #region Variables declaration
        private Bitmap image;
        private List<Point> diodesPositionList;
        #endregion

        #region Methods declaration
        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="imageTemp"></param>
        /// <param name="diodesPositionListTemp"></param>
        public OnCameraFingerChangedDebugEventArgs(Image<Bgr, Byte> imageTemp, List<Point> diodesPositionListTemp)
        {
         imageTemp =   imageTemp.Flip(Emgu.CV.CvEnum.FLIP.HORIZONTAL);
         imageTemp = imageTemp.Flip(Emgu.CV.CvEnum.FLIP.VERTICAL) ;
            image = imageTemp.ToBitmap();
            diodesPositionList = diodesPositionListTemp;
        }
        #endregion

        #region Properties declaration

        /// <summary>
        /// Get the image after diodes tracking processing.
        /// </summary>
        public Bitmap Image
        {
            get {
                return image; }
        }

        /// <summary>
        /// Get the list of the positions of the fingers diodes tracked.
        /// </summary>
        public List<Point> DiodesPositionList
        {
            get { return diodesPositionList; }
        }
        #endregion

    }

    /// <summary>
    /// Frames found event class.
    /// </summary>
    public class OnCameraFramesFoundEventArgs : EventArgs
    {
        #region Variables declaration
        #endregion

        #region Methods declaration
        #endregion

        #region Properties declaration
        #endregion
    }

    /// <summary>
    /// Frames lost event class.
    /// </summary>
    public class OnCameraFramesLostEventArgs:EventArgs
    {
        #region Variables declaration
        #endregion

        #region Methods declaration
        #endregion

        #region Properties declaration
        #endregion
    }

    /// <summary>
    /// Frames diodes position changed event class.
    /// </summary>
    public class OnCameraFramesChangedEventArgs : EventArgs
    {
        #region Variables declaration
        private List<PointF> framesDiodesPositionList;
        #endregion

        #region Methods declaration
        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="framesDiodesPositionListTemp"></param>
        public OnCameraFramesChangedEventArgs(List<PointF> framesDiodesPositionListTemp)
        {
            framesDiodesPositionList = framesDiodesPositionListTemp;
        }
        #endregion

        #region Properties declaration
        /// <summary>
        /// Get the list of the positions of the frames diodes tracked.
        /// </summary>
        public List<PointF> FramesDiodesPositionList
        {
            get { return framesDiodesPositionList; }
                    }
        #endregion
    }

       
}
